<?php
/**
 * @package     itoctopus
 * @subpackage  Search.files
 * @copyright   Copyright (C) 2015 itoctopus ( http://www.itoctopus.com ). All rights reserved.
 * @description This plugin will search through all the text, HTML, doc, docx, pptx, xlsx, and PDF files in a specific directory for a keyword supplied by the user and will return all the matching files. It's better to order this plugin last in your search plugins. Developed by itoctopus - http://www.itoctopus.com .
 * @author   	itoctopus 
 * @date   		10:30 AM 15/11/2015
 * @location   	Montreal, QC, Canada
 * @license     Free
 * @disclaimer	This plugin is provided as is and comes with no warrany whatsoever. You cannot hold itoctopus, its affiliates, or any of its developers liable for any issue resulting from the use of this plugin, whether that issue is directly or indirectly caused by this plugin. You are using this plugin at your own risk.
 */
 

defined('_JEXEC') or die;

/**
 * Files search plugin.
 *
 */
class PlgSearchFiles extends JPlugin
{

	/* 
	 *	@func	onContentSearch
	 *	@author	itoctopus - http://www.itoctopus.com
	 *	@desc	This function will search the content of the files in a specific directory. Currently, the plugin can search text files, HTML files, PDF files, MS document files (doc and docx files), MS Excel files (xlsx files only), and MS PowerPoint files (pptx files only)
	 *	@param  string  $text      Target search string.
	 *	@param	string  $phrase    [ignored] Matching option (possible values: exact|any|all).  Default is "any".
	 *	@param	string  $ordering  [ignored] Ordering option (possible values: newest|oldest|popular|alpha|category).  Default is "newest".
	 *	@param	mixed   $areas     [ignored] An array if the search it to be restricted to areas or null to search all areas.
	 *	@return	array  Search results consisting of file name, file links, and excerpt of these files.
	 *	@date	15/11/2015
	*/
	 
	public function onContentSearch($text, $phrase = '', $ordering = '', $areas = null){
		$results = array();
		$i = 0;
		//add the forward slash if it doesn't exist
		$unalteredSearchDirectory = $this->params->def('search_directory', 'docs');
		if ($unalteredSearchDirectory[0] != '/')
			$searchDirectory = '/'.$unalteredSearchDirectory;
		
		//now remove the document root if it's already added to the searchDirectory
		$searchDirectory = str_replace($_SERVER['DOCUMENT_ROOT'], '', $searchDirectory);
		
		//now add the document root
		$searchDirectory = $_SERVER['DOCUMENT_ROOT'].'/'.$searchDirectory;
		
		//now let's replace the // with a single slash
		$searchDirectory = str_replace('//', '/', $searchDirectory);
		
		//now let's check if the directory exists - if it doesn't, then we fail silently
		if (!is_dir($searchDirectory))
			return $results;
		
		//the directory exists, let's get all the files within that directory and search them
		$objRDI = new RecursiveDirectoryIterator($searchDirectory);
		
		//require both the PDF library and the MS library
		require_once(dirname(__FILE__).'/helpers/files.ms2text.php');
		require_once(dirname(__FILE__).'/helpers/files.pdf2text.php');
		
		foreach (new RecursiveIteratorIterator($objRDI) as $objSplFile) {
			if (!$objSplFile->isFile())
				continue;
			$filePath 				= $objSplFile->getRealPath();
			$fileName 				= $objSplFile->getFilename();
			$fileExtension 			= strtolower($objSplFile->getExtension());
			$fileLastModifiedTime 	= $objSplFile->getCTime();
			//filter out all files that are not txt, text, htm, html, pdf, doc, docx, xlsx and pptx
			if (!in_array ($fileExtension, array('txt', 'text', 'htm', 'html', 'pdf', 'doc', 'docx', 'xlsx', 'pptx')))
				continue;
			
			//if this is a PDF file, then let's instantiate a FilesPDF2Text class to convert it to text
			if ($fileExtension == 'pdf'){
				$objFilesPDF2Text = new FilesPDF2Text();
				$objFilesPDF2Text->setFilename($filePath);
				$objFilesPDF2Text->decodePDF();
				$strFileContentsRaw	= $objFilesPDF2Text->output();
			}
			elseif (in_array ($fileExtension, array('doc', 'docx', 'xlsx', 'pptx'))){
				$objFilesMS2Text	= new FilesMS2Text($filePath);
				$strFileContentsRaw	= $objFilesMS2Text->convertToText();
			}
			else{
				$strFileContentsRaw	= file_get_contents ($filePath);				
			}
			
			
			
			//let's get the content of the file and check if the keyword exists in the content
			//remove all HTML from the files
		    $strFileContentsNoTags = strip_tags($strFileContentsRaw);
		    $strFileContents = strtolower($strFileContentsNoTags);
		    if (stripos ($strFileContents , $text) !== FALSE){
		    	$results[$i] = new stdClass();
		    	$results[$i]->title 	= $fileName;
		    	$results[$i]->href 		= JURI::root().$filePath;
		    	if ($_SERVER['DOCUMENT_ROOT'] != '' && $_SERVER['DOCUMENT_ROOT'] != '/')
		    		$results[$i]->href 	= str_replace($_SERVER['DOCUMENT_ROOT'].'/', '', $results[$i]->href);
		    	$results[$i]->text 		= $strFileContentsNoTags;
		    	$results[$i]->section 	= 'Files / '.$unalteredSearchDirectory;
		    	$results[$i]->created 	= $fileLastModifiedTime; //not really the created time, but the last modified time. Linux does not record the created time.
		    	$i++;
		    }
		}
		return $results;
	}
}