<?php

/*
 * @name		FilesMS2Text
 * @desc 		This PHP class converts a Microsoft doc, docx (Word), pptx (PowerPoint), xlsx (Excel) to text
 * @desc		This is a helper file for the itoctopus "Files - Search" plugin that is used to search Microsoft Office documents
 * @author 		Gourav Mehta ( gouravmehta@gmail.com ) and itoctopus ( development@itoctopus.com )
 * @note		itoctopus is not associated with Gourav Mehta - Gourav Mehta is the original author of this class which itoctopus modified to suit the needs of the "Files - Search" plugin.
 * @location 	Montreal, QC, Canada
 * @created		November 2015
 * @license     GPL
 * @disclaimer  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

defined('_JEXEC') or die;


class FilesMS2Text{
	private $filename;

	public function __construct($filePath) {
		$this->filename = $filePath;
	}

	private function readDoc() {
		$fileHandle = fopen($this->filename, "r");
		$line = @fread($fileHandle, filesize($this->filename));
		$lines = explode(chr(0x0D),$line);
		$outtext = "";
		foreach($lines as $thisline){
			$pos = strpos($thisline, chr(0x00));
			if (($pos !== FALSE)||(strlen($thisline)==0)){
			} else {
				$outtext .= $thisline." ";
			}
		}
		$outtext = preg_replace("/[^a-zA-Z0-9\s\,\.\-\n\r\t@\/\_\(\)]/","",$outtext);
		return $outtext;
	}

	private function readDocX(){
		$striped_content = '';
		$content = '';
		$zip = zip_open($this->filename);
		if (!$zip || is_numeric($zip)) return false;
		while ($zip_entry = zip_read($zip)) {
			if (zip_entry_open($zip, $zip_entry) == FALSE) continue;
			if (zip_entry_name($zip_entry) != "word/document.xml") continue;
			$content .= zip_entry_read($zip_entry, zip_entry_filesize($zip_entry));
			zip_entry_close($zip_entry);
		}
		zip_close($zip);
		$content = str_replace('</w:r></w:p></w:tc><w:tc>', " ", $content);
		$content = str_replace('</w:r></w:p>', "\r\n", $content);
		$striped_content = strip_tags($content);
		return $striped_content;
	}
	
	function readXLSX(){
		$xml_filename = "xl/sharedStrings.xml"; //content file name
		$zip_handle = new ZipArchive;
		$output_text = "";
		if(true === $zip_handle->open($this->filename)){
			if(($xml_index = $zip_handle->locateName($xml_filename)) !== false){
				$xml_datas = $zip_handle->getFromIndex($xml_index);
				$xml_handle = DOMDocument::loadXML($xml_datas, LIBXML_NOENT | LIBXML_XINCLUDE | LIBXML_NOERROR | LIBXML_NOWARNING);
				$output_text = strip_tags($xml_handle->saveXML());
			}else{
				$output_text .="";
			}
			$zip_handle->close();
		}else{
			$output_text .="";
		}
		return $output_text;
	}

	function readPPTX(){
		$zip_handle = new ZipArchive;
		$output_text = "";
		if(true === $zip_handle->open($this->filename)){
			$slide_number = 1;
			while(($xml_index = $zip_handle->locateName("ppt/slides/slide".$slide_number.".xml")) !== false){
				$xml_datas = $zip_handle->getFromIndex($xml_index);
				$xml_handle = DOMDocument::loadXML($xml_datas, LIBXML_NOENT | LIBXML_XINCLUDE | LIBXML_NOERROR | LIBXML_NOWARNING);
				$output_text .= strip_tags($xml_handle->saveXML());
				$slide_number++;
			}
			if($slide_number == 1){
				$output_text .="";
			}
			$zip_handle->close();
		}else{
			$output_text .="";
		}
		return $output_text;
	}


	public function convertToText() {
		if(isset($this->filename) && !file_exists($this->filename)) {
			return "File Not exists";
		}
		$fileArray = pathinfo($this->filename);
		$file_ext  = $fileArray['extension'];
		if($file_ext == "doc" || $file_ext == "docx" || $file_ext == "xlsx" || $file_ext == "pptx"){
			if($file_ext == "doc") {
				return $this->readDoc();
			} elseif($file_ext == "docx") {
				return $this->readDocX();
			} elseif($file_ext == "xlsx") {
				return $this->readXLSX();
			}elseif($file_ext == "pptx") {
				return $this->readPPTX();
			}
		} else {
			return "Invalid File Type";
		}
	}
}
?>